{   Component(s):
    tcySpeedButton

    Description:
    Simple SpeedButton with color feature

    * ***** BEGIN LICENSE BLOCK *****
    *
    * Version: MPL 1.1
    *
    * The contents of this file are subject to the Mozilla Public License Version
    * 1.1 (the "License"); you may not use this file except in compliance with the
    * License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
    *
    * Software distributed under the License is distributed on an "AS IS" basis,
    * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
    * the specific language governing rights and limitations under the License.
    *
    * The Initial Developer of the Original Code is Mauricio
    * (https://sourceforge.net/projects/tcycomponents/).
    *
    * Donations: see Donation section on Description.txt
    *
    * Alternatively, the contents of this file may be used under the terms of
    * either the GNU General Public License Version 2 or later (the "GPL"), or the
    * GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in which
    * case the provisions of the GPL or the LGPL are applicable instead of those
    * above. If you wish to allow use of your version of this file only under the
    * terms of either the GPL or the LGPL, and not to allow others to use your
    * version of this file under the terms of the MPL, indicate your decision by
    * deleting the provisions above and replace them with the notice and other
    * provisions required by the LGPL or the GPL. If you do not delete the
    * provisions above, a recipient may use your version of this file under the
    * terms of any one of the MPL, the GPL or the LGPL.
    *
    * ***** END LICENSE BLOCK *****}

unit cySpeedButton;

{$MODE Delphi}

{$I cyCompilerDefines.inc}

interface

uses LCLIntf, LCLType, LMessages,
      Graphics, Classes, Themes, Controls, {$IFDEF DELPHI2009_OR_ABOVE} pngimage, {$ENDIF}
      Buttons, cyTypes, cyClasses, cyGraphics, cyBaseSpeedButton;

type
  TcyCustomSpeedButton = class;

  // Use TImageList instead of Glyph:
  TcyImagelistOptions = class(TPersistent)
  private
    FOwner: TcyCustomSpeedButton;
    FImageList: TImageList;
    FDisabledIndex: Integer;
    FMouseDownIndex: Integer;
    FExclusiveIndex: Integer;
    FNormalIndex: Integer;
    procedure SetDisabledIndex(const Value: Integer);
    procedure SetExclusiveIndex(const Value: Integer);
    procedure SetImageList(const Value: TImageList);
    procedure SetMouseDownIndex(const Value: Integer);
    procedure SetNormalIndex(const Value: Integer);
  protected
  public
    constructor Create(AOwner: TComponent); virtual;
    procedure GetDrawingParams(State: TButtonState; var ImageIndexParam: Integer; var EnabledParam: Boolean);
  published
    property DisabledIndex: Integer read FDisabledIndex write SetDisabledIndex default -1;
    property ExclusiveIndex: Integer read FExclusiveIndex write SetExclusiveIndex default -1;
    property MouseDownIndex: Integer read FMouseDownIndex write SetMouseDownIndex default -1;
    property NormalIndex: Integer read FNormalIndex write SetNormalIndex default -1;
    property ImageList: TImageList read FImageList write SetImageList;
  end;

  TcyCustomSpeedButton = class(TcyBaseSpeedButton)
  private
    FDegrade: TcyGradient;
    FHotFromColor: TColor;
    FHotToColor: TColor;
    FHotBalanceMode: TDgradBalanceMode;
    FDownFromColor: TColor;
    FDownToColor: TColor;
    FDisabledFromColor: TColor;
    FDisabledToColor: TColor;
    FImagelistOptions: TcyImagelistOptions;
    FWallpaper: TcyBgPicture;
    procedure SetDegrade(const Value: TcyGradient);
    procedure SetImagelistOptions(const Value: TcyImagelistOptions);
    procedure SetWallpaper(const Value: TcyBgPicture);
  protected
    procedure DrawButton(aState: TButtonState; Hot: Boolean); override;
    procedure DrawBackground(var Rect: TRect; aState: TButtonState; Hot: Boolean); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DegradeChanged(Sender: TObject);
    procedure WallpaperChanged(Sender: TObject);
    // To be public :
    property DefaultHotFromColor: TColor read FHotFromColor write FHotFromColor;
    property DefaultHotToColor: TColor read FHotToColor write FHotToColor;
    property DefaultDownFromColor: TColor read FDownFromColor write FDownFromColor;
    property DefaultDownToColor: TColor read FDownToColor write FDownToColor;
    property DefaultDisabledFromColor: TColor read FDisabledFromColor write FDisabledFromColor;
    property DefaultDisabledToColor: TColor read FDisabledToColor write FDisabledToColor;
    // To be published :
    property Degrade: TcyGradient read FDegrade write SetDegrade;
    property ImagelistOptions: TcyImagelistOptions read FImagelistOptions write SetImagelistOptions;
    property Wallpaper: TcyBgPicture read FWallpaper write SetWallpaper;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
  end;

  TcySpeedButton = class(TcyCustomSpeedButton)
  private
  protected
  public
    property MouseOver;
    // Herited from TcyCustomSpeedButton :
    property DefaultHotFromColor;
    property DefaultHotToColor;
    property DefaultDownFromColor;
    property DefaultDownToColor;
    property DefaultDisabledFromColor;
    property DefaultDisabledToColor;
  published
    // Herited from TcyCustomSpeedButton :
    property Degrade;
    property ImagelistOptions;
    property Wallpaper;
    // Herited from TcyBaseSpeedButton :
    property CaptionOrientation;
    property FlatDownStyle;
    property FlatHotStyle;
    property GlyphX;
    property WordWrap;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnPaint;
  end;

const
  FFramePercent = -20;

implementation

{ TcyImagelistOptions }
constructor TcyImagelistOptions.Create(AOwner: TComponent);
begin
  FOwner := TcyCustomSpeedButton(AOwner);
  FDisabledIndex := -1;
  FExclusiveIndex := -1;
  FMouseDownIndex := -1;
  FNormalIndex := -1;
end;

procedure TcyImagelistOptions.SetDisabledIndex(const Value: Integer);
begin
  if FDisabledIndex <> Value
  then begin
    FDisabledIndex := Value;
    if Assigned(FImageList)
    then FOwner.Invalidate;
  end;
end;

procedure TcyImagelistOptions.SetExclusiveIndex(const Value: Integer);
begin
  if FExclusiveIndex <> Value
  then begin
    FExclusiveIndex := Value;
    if Assigned(FImageList)
    then FOwner.Invalidate;
  end;
end;

procedure TcyImagelistOptions.SetImageList(const Value: TImageList);
begin
  FImageList := Value;
  FOwner.Invalidate;

  if Value <> nil
  then Value.FreeNotification(FOwner);
end;

procedure TcyImagelistOptions.SetMouseDownIndex(const Value: Integer);
begin
  if FMouseDownIndex <> Value
  then begin
    FMouseDownIndex := Value;
    if Assigned(FImageList)
    then FOwner.Invalidate;
  end;
end;

procedure TcyImagelistOptions.SetNormalIndex(const Value: Integer);
begin
  if FNormalIndex <> Value
  then begin
    FNormalIndex := Value;
    if Assigned(FImageList)
    then FOwner.Invalidate;
  end;
end;

procedure TcyImagelistOptions.GetDrawingParams(State: TButtonState; var ImageIndexParam: Integer; var EnabledParam: Boolean);
begin
  EnabledParam := true;

  case State of
    bsUp, bsHot:   // 9999 RockyLuck
      ImageIndexParam := FNormalIndex;

    bsDown:
      ImageIndexParam := FMouseDownIndex;

    bsExclusive:
      ImageIndexParam := FExclusiveIndex;

    bsDisabled:
      begin
        ImageIndexParam := FDisabledIndex;
        EnabledParam := ImageIndexParam > -1;
      end;
  end;

  if ImageIndexParam = -1
  then ImageIndexParam := FNormalIndex;
end;

{ TcyCustomSpeedButton }
constructor TcyCustomSpeedButton.Create(AOwner: TComponent);
begin
  inherited;
  FDegrade := TcyGradient.Create(Self);
  FDegrade.OnChange := DegradeChanged;
  FDegrade.FromColor := clYellow;
  FWallpaper := TcyBgPicture.Create(self);
  FWallpaper.OnChange := WallpaperChanged;
  FDegrade.ToColor := $000080FF;  // Orange color ...
  FDegrade.SpeedPercent := 90;
  FHotBalanceMode := bmReverseFromColor;
  DegradeChanged(Self);
  FImagelistOptions := TcyImagelistOptions.Create(Self);
end;

destructor TcyCustomSpeedButton.Destroy;
begin
  FWallpaper.Free;
  FDegrade.Free;
  FImagelistOptions.Free;
  inherited;
end;

procedure TcyCustomSpeedButton.SetImagelistOptions(const Value: TcyImagelistOptions);
begin
  FImagelistOptions.Assign(Value);
end;

procedure TcyCustomSpeedButton.SetDegrade(const Value: TcyGradient);
begin
  FDegrade := Value;
end;

procedure TcyCustomSpeedButton.SetWallpaper(const Value: TcyBgPicture);
begin
  FWallpaper := Value;
end;

procedure TcyCustomSpeedButton.WallpaperChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TcyCustomSpeedButton.DegradeChanged(Sender: TObject);
begin
  if FDegrade.FromColor = FDegrade.ToColor
  then begin
    FHotFromColor := ColorSetPercentBrightness(FDegrade.FromColor, 20);
    FHotToColor := FHotFromColor;
  end
  else begin
    FHotFromColor := FDegrade.FromColor;
    FHotToColor := FDegrade.ToColor;
  end;

  FDownFromColor := ColorSetPercentBrightness(FDegrade.FromColor, -20);
  FDownToColor := ColorSetPercentBrightness(FDegrade.ToColor, -20);
  FDisabledFromColor := ColorSetPercentBrightness(FDegrade.FromColor, 30);
  FDisabledToColor := ColorSetPercentBrightness(FDegrade.ToColor, 30);
  Invalidate;
end;

procedure TcyCustomSpeedButton.DrawButton(aState: TButtonState; Hot: Boolean);
var
  ImageHandled: Boolean;
  Rect, TextRect: TRect;
  aGraphicWidth, aGraphicHeight, aGraphicX, aGraphicY: Integer;
  ImageListIndex: Integer;
  ImageListEnabled: Boolean;
begin
  Rect := ClientRect;
  ImageHandled := false;
  DrawBackground(Rect, aState, Hot);

  // Draw caption/glyph :
  Canvas.Font := Self.Font;

  if Assigned(FImagelistOptions.FImageList)
  then begin
    FImagelistOptions.GetDrawingParams(aState, ImageListIndex, ImageListEnabled);

    if ImageListIndex > -1
    then begin
      aGraphicWidth := FImagelistOptions.FImageList.Width;
      aGraphicHeight := FImagelistOptions.FImageList.Height;
      CalcLayout(Rect, aGraphicWidth, aGraphicHeight, aGraphicX, aGraphicY, TextRect);
      FImagelistOptions.FImageList.Draw(Canvas, aGraphicX, aGraphicY, ImageListIndex, ImageListEnabled);
      ImageHandled := true;
    end;
  end
  else
    if ValidGraphic(GlyphX.Graphic)
    then begin
      if GlyphX.Graphic is graphics.TBitmap
      then aGraphicWidth := GlyphX.Bitmap.Width div NumGlyphs
      else aGraphicWidth := GlyphX.Width;

      aGraphicHeight := GlyphX.Height;
      CalcLayout(Rect, aGraphicWidth, aGraphicHeight, aGraphicX, aGraphicY, TextRect);
      DrawGlyph(aGraphicX, aGraphicY, aState);
      ImageHandled := true;
    end;

  if not ImageHandled
  then begin
    aGraphicWidth := 0;
    aGraphicHeight := 0;
    CalcLayout(Rect, aGraphicWidth, aGraphicHeight, aGraphicX, aGraphicY, TextRect);
  end;

  Canvas.Brush.Style := bsClear;
  DrawCaption(TextRect, aState, Hot, aGraphicWidth <> 0);
  Canvas.Brush.Style := bsSolid;
end;

procedure TcyCustomSpeedButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;

  // Careful, FImageListOptions may be not created already, check operation first!
  if Operation = opRemove
  then
    if FImagelistOptions.FImageList <> nil
    then
      if AComponent = FImagelistOptions.FImageList
      then begin
        FImagelistOptions.FImageList := nil;
        Invalidate;
      end;
end;

procedure TcyCustomSpeedButton.DrawBackground(var Rect: TRect; aState: TButtonState; Hot: Boolean);
var
  fromColor, toColor, FrameColor, BorderLeftTopColor, BorderRightBottomColor: TColor;
  balanceMode: TDgradBalanceMode;
  Details: TThemedElementDetails;
begin
  // inherited;
  case aState of
    bsUp, bsHot:   // 9999 RockyLuck
      begin
        if Hot and (FlatHotStyle <> hsHidden)
        then begin
          fromColor := FHotFromColor;
          toColor := FHotToColor;
          balanceMode := FHotBalanceMode;
        end
        else begin
          fromColor := FDegrade.FromColor;
          toColor := FDegrade.ToColor;
          balanceMode := FDegrade.BalanceMode;
        end;

        BorderLeftTopColor := ColorSetPercentBrightness(fromColor, 40);
        BorderRightBottomColor := MediumColor(fromColor, ToColor);
        FrameColor := ColorSetPercentBrightness(toColor, FFramePercent);
      end;

    bsDisabled:
      begin
        fromColor := FDisabledFromColor;
        toColor := FDisabledToColor;
        balanceMode := FDegrade.BalanceMode;
        BorderLeftTopColor := ColorSetPercentBrightness(fromColor, 40);
        BorderRightBottomColor := MediumColor(fromColor, ToColor);
        FrameColor := ColorSetPercentBrightness(FDisabledToColor, FFramePercent);
      end;

    bsDown, bsExclusive:
      begin
        fromColor := FDownFromColor;
        toColor := FDownToColor;
        balanceMode := bmMirror;
        BorderLeftTopColor := MediumColor(fromColor, ToColor);
        BorderRightBottomColor := ColorSetPercentBrightness(fromColor, 40);
        FrameColor := ColorSetPercentBrightness(FDownToColor, FFramePercent);
      end;
  end;

  // Draw button body :
  if (not Flat) or (not Transparent)
  then cyGradientFill(Canvas, Rect, fromColor, toColor,  FDegrade.Orientation, FDegrade.Balance,
                       balanceMode, FDegrade.MaxDegrade, FDegrade.SpeedPercent);

  // Draw Wallpaper :
  cyDrawBgPicture(Canvas, Rect, FWallpaper);

  // Draw frame :
  if Flat
  then begin
    if (aState = bsUp) and Hot     // Show hot state on flat buttons :
    then begin
      // if ThemeControl(Self) and (FlatHotStyle = hsWindowsTheme)
      if ThemeServices.ThemesEnabled and (FlatHotStyle = hsWindowsTheme)
      then begin
        Details := ThemeServices.GetElementDetails(ttbButtonHot);
        ThemeServices.DrawElement(Canvas.Handle, Details, Rect);
        Rect := ThemeServices.ContentRect(Canvas.Handle, Details, Rect);
      end
      else
        if FlatHotStyle <> hsHidden
        then DrawBorders(Rect, BorderLeftTopColor, ColorSetPercentBrightness(BorderRightBottomColor, -40));
    end;

    if aState in [bsDown, bsExclusive]           // Show down state on flat buttons :
    then begin
      // if ThemeControl(Self) and (FlatDownStyle = dsWindowsTheme)
      if ThemeServices.ThemesEnabled and (FlatDownStyle = dsWindowsTheme)
      then begin
        Details := ThemeServices.GetElementDetails(ttbButtonPressed);
        ThemeServices.DrawElement(Canvas.Handle, Details, Rect);
        Rect := ThemeServices.ContentRect(Canvas.Handle, Details, Rect);
      end
      else
        if FlatDownStyle <> dsHidden
        then DrawBorders(Rect, ColorSetPercentBrightness(BorderLeftTopColor, -40), BorderRightBottomColor);
    end;
  end
  else begin
    DrawBorders(Rect, FrameColor, FrameColor);

    if aState = bsDown
    then DrawBorders(Rect, BorderLeftTopColor, BorderRightBottomColor)
    else DrawInnerBorders(Rect, BorderLeftTopColor, BorderRightBottomColor);
  end;
end;

end.
